from typing import List, Dict


class BenefitList(object):
    def __init__(
        self,
        benefit_pre_total_sequence,     # type: List
        persisted_benefits,             # type: List
        discount_item_values,           # type: Dict
        discount_id_tip_excluded,       # type: List
        tender_id_tip_excluded,         # type: List
        benefits,                       # type: str
    ):
        # type: (...) -> None

        self.benefit_pre_total_sequence = benefit_pre_total_sequence
        self.persisted_benefits = persisted_benefits
        self.discount_item_values = discount_item_values
        self.discount_id_tip_excluded = discount_id_tip_excluded
        self.tender_id_tip_excluded = tender_id_tip_excluded
        self.benefit_list = benefits
