from report.command import Command  # noqa
from typing import List, Optional  # noqa


class Part(object):
    def __init__(
        self,
        text=None,          # type: Optional[str]
        commands=None,      # type: Optional[List[Command]]
        encoding='ASCII',   # type: Optional[str]
    ):
        # type: (...) -> None

        self.text = text
        self.commands = commands
        self.encoding = encoding
