from messagebus import TokenCreator, TokenPriority


class LoyaltyToken(object):
    LOYALTY_MESSAGE_GROUP = "111"

    @staticmethod
    def create_token(
        number,  # type: str
    ):
        return TokenCreator.create_token(
            priority=TokenPriority.low,
            message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
            number=number
        )
