from typing import Optional


class CatalogVersionDto(object):
    def __init__(
        self,
        menu_id,        # type: Optional[str]
        menu_version,   # type: Optional[str]
        menu_name,      # type: Optional[str]
    ):
        # type: (...) -> None

        self.menu_id = menu_id
        self.menu_version = menu_version
        self.menu_name = menu_name
