from typing import Optional


class ElectronicMessage(object):

    def __init__(self, pos_id, event_type, timestamp, message_id=None, title=None, message_data=None, code_type=None,
                 input_type=None, choices=None, buttons=None):
        # type: (int, str, str, Optional[int], Optional[str], Optional[str], Optional[str], Optional[str], Optional[str], Optional[str]) -> None

        self.pos_id = pos_id
        self.event_type = event_type
        self.timestamp = timestamp
        self.message_id = message_id
        self.title = title
        self.message_data = message_data
        self.code_type = code_type
        self.input_type = input_type
        self.choices = choices
        self.buttons = buttons
