from model.log import LogEntry


class EventDispatch(object):
    def __init__(self, event_time, row_id, log_entry, encoded=None):
        # type: (str, int, LogEntry, bool) -> None
        self.event_time = event_time
        self.row_id = row_id
        self.log_entry = log_entry
        self.encoded = encoded
