from abc import (
    ABCMeta,
    abstractmethod,
)

from posot import OrderTaker


class LoyaltyServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_customer_info(
        self,
        pos_ot,         # type: OrderTaker
        customer_id,    # type: str
    ):
        # type: (...) -> SearchCustomerInfoResponse

        raise NotImplementedError
