from ._LoyaltyToken import LoyaltyToken
from messagebus import TokenCreator, TokenPriority


class LoyaltyRequestToken(LoyaltyToken):
    TK_LOYALTY_GET_AND_LOCK_VOUCHER = LoyaltyToken.create_token("2")
    TK_LOYALTY_BURN_VOUCHER = LoyaltyToken.create_token("3")
    TK_LOYALTY_UNLOCK_VOUCHER = LoyaltyToken.create_token("4")
    TK_LOYALTY_LOCK_BENEFIT = LoyaltyToken.create_token("6")
