from typing import List

from devicemanager.interactor.dto import DeviceStatusDto
from devicemanager.service import GetDevicesService


class GetDevicesInteractor(object):
    def __init__(
        self,
        get_devices_service,  # type: GetDevicesService
    ):
        # type: (...) -> None

        self.get_devices_service = get_devices_service

    def get_devices(
        self,  # type: GetDevicesInteractor
    ):
        # type: (...) -> List[DeviceStatusDto]

        return self.get_devices_service.get_devices()
