from typing import (
    Dict,
    Optional,
)


class OrderDiscountsDetail(object):
    def __init__(
        self,
        discount_ids=None,         # type: Optional[Dict]
        discount_amounts=None,     # type: Optional[Dict]
    ):
        # type: (...) -> None

        self.discount_ids = discount_ids if discount_ids else dict()
        self.discount_amounts = discount_amounts if discount_amounts else dict()
