from sysactions import (
    show_any_dialog, action,
)

@action
def do_start_network_test(
    pos_id,  # type: str
):
    # type: (...) -> None
    show_any_dialog(
        pos_id,
        "networkTest",
        "",
        "",
        "",
        "",
        "",
        60000,
        "",
        "",
        "",
        "",
        "",
        True
    )
