from _StatusResponse import StatusResponse


class GetStoreResponse(object):
    def __init__(
        self,
        menu_id,        # type: str
        menu_version,   # type: str
        menu_name,      # type: str
        printer,        # type: StatusResponse
        eft,            # type: StatusResponse
        fiscal,         # type: StatusResponse
        manager,        # type: StatusResponse
    ):
        # type: (...) -> None

        self.menu_id = menu_id
        self.menu_version = menu_version
        self.menu_name = menu_name
        self.printer = printer
        self.eft = eft
        self.fiscal = fiscal
        self.manager = manager
